@@ -631,7 +631,7 @@ PyObject * quisk_freedv_set_options(PyObject * self, PyObject * args, PyObject *
 	if (!PyArg_ParseTupleAndKeywords (args, keywds, "|isiiii", kwlist, &mode, &ptMsg, &DEBUG, &quisk_freedv_squelch, &interleave_frames, &bpf))
 		return NULL;
 	if (ptMsg)
-		strncpy(quisk_tx_msg, ptMsg, TX_MSG_SIZE);
+		strMcpy(quisk_tx_msg, ptMsg, TX_MSG_SIZE);
 	if (bpf != -1) {
 		quisk_set_tx_bpf = bpf;
 		if (freedv_set_tx_bpf && rx_channel[0].hFreedv)
