@@ -29,10 +29,6 @@ class Hardware(BaseHardware):
     self.hermes_rev_power = 0.0
     self.hermes_pa_current = 0.0
     self.eeprom_valid = 0
-    self.alex_hpf_f1 = 0
-    self.alex_hpf_f2 = 0
-    self.alex_lpf_f1 = 0
-    self.alex_lpf_f2 = 0
     self.mode = None
     self.band = None
     self.vfo_frequency = 0
@@ -40,6 +36,9 @@ class Hardware(BaseHardware):
     self.vna_count = 0
     self.vna_started = False
     self.repeater_freq = None		# original repeater output frequency
+    self.antenna_labels = ('Ant 0', 'Ant 1')	# labels for the antenna button
+    self.antenna_index = 0			# index of antenna in use
+    self.delay_config = 30			# Delay sending message to HL2 until after sound starts
     try:
       self.repeater_delay = conf.repeater_delay		# delay for changing repeater frequency in seconds
     except:
@@ -58,14 +57,20 @@ class Hardware(BaseHardware):
     if DEBUG: print ('broadcast_addrs', self.broadcast_addrs)
     # This is the control data to send to the Hermes using the Metis protocol
     # Duplex must be on or else the first Rx frequency is locked to the Tx frequency
+    #
+    # From the protocol document, the data are: C1==[31:24], C2=[23:16], C3=[15:8], C4=[7:0] where [] are inclusive.
+    # self.pc2hermes[] is used for the first 17 addresses 0x00 through 0x10. These are sent round-robin.
     self.pc2hermes = bytearray(17 * 4)		# Control bytes not including C0.  Python initializes this to zero.
-    self.pc2hermeslitewritequeue = bytearray(4 * 5)
+    # Addresses C0 = 0x12 through 0x3E use self.pc2hermeswritequeue, which is capable of requesting an ACK.
+    #    C0 is a six-bit address. If the address is OR'd with 0x40, an ACK must be received or the item is re-transmitted.
+    #    The item sent to the HL2 is C0<<1 | MoxBit.
+    #    Sound must be started before you can use pc2hermeswritequeue.
+    self.pc2hermeslitewritequeue = bytearray(4 * 5)	# Four of (C0, C1, C2, C3, C4)
+    # Initialize some data
     self.pc2hermes[3] = 0x04	# C0 index == 0, C4[5:3]: number of receivers 0b000 -> one receiver; C4[2] duplex on
     self.pc2hermes[4 * 9] = 63	# C0 index == 0b1001, C1[7:0] Tx level
     for c0 in range(1, 9):		# Set all frequencies to 7012352, 0x006B0000
       self.SetControlByte(c0, 2, 0x6B)
-    self.SetLowPwrEnable(conf.hermes_lowpwr_tr_enable)
-    self.EnablePowerAmp(conf.hermes_power_amp)
     self.ChangeTxLNA(conf.hermes_TxLNA_dB)
     self.MakePowerCalibration()
   def pre_open(self):
@@ -170,6 +175,10 @@ class Hardware(BaseHardware):
     self.config_text = st
     self.ChangeLNA(2)	# Initialize the LNA using the correct LNA code from the FPGA code version
   def open(self):
+    self.delay_config = 30			# Delay sending message to HL2 until after sound starts
+    for name in ('hermes_tx_buffer_latency', 'keyupDelay',
+        'hermes_lowpwr_tr_enable', 'hermes_PWM', 'hermes_disable_sync', 'hermes_power_amp', 'Hermes_BandDictEnTx'):
+      self.ImmediateChange(name)
     return self.config_text
   def GetValue(self, name):	# return values stored in the hardware
     if name == 'Hware_Hl2_EepromIP':
@@ -259,14 +268,32 @@ class Hardware(BaseHardware):
     # Get the control byte at C0 index and byte index.  The bytes are C0, C1, C2, C3, C4.
     # The C0 index is 0 to 16 inclusive.  The byte index is 1 to 4.  The byte index of C2 is 2.
     return self.pc2hermes[C0_index * 4 + byte_index - 1]
-  def SetControlByte(self, C0_index, byte_index, value):		# Set the control byte as above.
+  def SetControlByte(self, C0_index, byte_index, value, prnt=True):		# Set the control byte as above.
     self.pc2hermes[C0_index * 4 + byte_index - 1] = value
     QS.pc_to_hermes(self.pc2hermes)
-    if DEBUG: print ("SetControlByte C0_index %d byte_index %d to 0x%X" % (C0_index, byte_index, value))
+    if DEBUG and prnt: print ("SetControlByte C0_index 0x%X byte_index %d to 0x%X" % (C0_index, byte_index, value))
+  def GetControlBit(self, C0_index, bit):
+    # Get the control bit (return 0 or 1) at C0 index and bit number 0 through 31.
+    byte_index = 4 - bit // 8
+    byte_value =  self.pc2hermes[C0_index * 4 + byte_index - 1]
+    mask = 0x01 << (bit % 8)
+    if byte_value & mask:
+      return 1
+    return 0
+  def SetControlBit(self, C0_index, bit, bit_value):
+    # Set the control bit at C0 index and bit number 0 through 31 to value (0 or 1).
+    byte_index = 4 - bit // 8
+    byte_value =  self.pc2hermes[C0_index * 4 + byte_index - 1]
+    mask = 0x01 << (bit % 8)
+    if bit_value:	# set bit to one
+      byte_value |= mask
+    else:		# set bit to zero
+      byte_value &= ~mask
+    self.pc2hermes[C0_index * 4 + byte_index - 1] = byte_value
+    QS.pc_to_hermes(self.pc2hermes)
+    #if DEBUG: print ("SetControlBit C0_index 0x%X byte_index %d to 0x%X" % (C0_index, byte_index, byte_value))
   def ChangeFrequency(self, tx_freq, vfo_freq, source='', band='', event=None):
     if tx_freq and tx_freq > 0:
-      if source == 'BtnBand' or abs(tx_freq - self.tx_frequency) > 1000000:
-        self.ChangeAlexFilters(tx_freq=tx_freq)
       self.tx_frequency = tx_freq
       tx = int(tx_freq - self.transverter_offset)
       self.pc2hermes[ 4] = tx >> 24 & 0xff		# C0 index == 1, C1, C2, C3, C4: Tx freq, MSB in C1
@@ -283,47 +310,6 @@ class Hardware(BaseHardware):
     if DEBUG > 1: print("Change freq Tx", tx_freq, "Rx", vfo_freq)
     QS.pc_to_hermes(self.pc2hermes)
     return tx_freq, vfo_freq
-  def ChangeAlexFilters(self, tx_freq=None, edit=False):
-    if tx_freq is None:
-      tx_freq = self.tx_frequency
-    fmin = tx_freq
-    fmax = tx_freq
-    for pane in self.application.multi_rx_screen.receiver_list:
-      freq = pane.VFO + pane.txFreq
-      if freq < fmin:
-        fmin = freq
-      if freq > fmax:
-        fmax = freq
-    if DEBUG: print ('fmin', fmin, 'fmax', fmax)
-    if edit or not (self.alex_hpf_f1 <= fmin < self.alex_hpf_f2):    # Within same HP filter?
-      self.alex_hpf_f1, self.alex_hpf_f2, rx, tx = self.FreqAlexFilters(fmin, self.conf.AlexHPF, self.conf.AlexHPF_TxEn)
-      if DEBUG: print ("Change HP filter fmin, f1, f2", fmin, self.alex_hpf_f1, self.alex_hpf_f2, "rx, tx", rx, tx)
-      QS.set_alex_hpf(rx, tx)
-    if edit or not (self.alex_lpf_f1 <= fmax < self.alex_lpf_f2):    # Within same LP filter?
-      self.alex_lpf_f1, self.alex_lpf_f2, rx, tx = self.FreqAlexFilters(fmax, self.conf.AlexLPF, self.conf.AlexLPF_TxEn)
-      if DEBUG: print ("Change LP filter fmax, f1, f2", fmax, self.alex_lpf_f1, self.alex_lpf_f2, "rx, tx", rx, tx)
-      QS.set_alex_lpf(rx, tx)
-  def FreqAlexFilters(self, tx_freq, filt, enabl):
-    # Find the new frequency band
-    gap1 = 0.0        # If we are in a gap in the filter frequencies, this is f1 and f2 for the gap.
-    gap2 = 1E20
-    for f1, f2, rx, tx in filt:  # f1 and f2 are strings in MHz
-      try:
-        f1 = float(f1) * 1E6
-        f2 = float(f2) * 1E6
-      except:
-        continue
-      if f1 >= f2:
-        continue
-      if f1 <= tx_freq < f2:
-        if not enabl:
-          tx = rx
-        return f1, f2, rx, tx
-      if tx_freq >= f2 and f2 > gap1:
-        gap1 = f2
-      if tx_freq <= f1 and f1 < gap2:
-        gap2 = f1
-    return gap1, gap2, 0, 0
   def Freq2Phase(self, freq=None):		# Return the phase increment as calculated by the FPGA
     # This code attempts to duplicate the calculation of phase increment in the FPGA code.
     clock = ((int(self.conf.rx_udp_clock) + 24000) // 48000) * 48000		# this assumes the nominal clock is a multiple of 48kHz
@@ -345,6 +331,11 @@ class Hardware(BaseHardware):
     self.hermes_temperature, self.hermes_fwd_power, self.hermes_rev_power, self.hermes_pa_current = QS.get_hermes_TFRC()
     if self.application.bottom_widgets:
       self.application.bottom_widgets.UpdateText()
+    if self.delay_config > 0:
+      self.delay_config -= 1
+      if self.delay_config == 0:	# Delay sending these config parameters
+        for name in ('hermes_disable_watchdog', 'hermes_reset_on_disconnect'):
+          self.ImmediateChange(name)
   def RepeaterOffset(self, offset=None):	# Change frequency for repeater offset during Tx
     if offset is None:		# Return True if frequency change is complete
       if time.time() > self.repeater_time0 + self.repeater_delay:
@@ -366,7 +357,12 @@ class Hardware(BaseHardware):
     self.band = band
     self.ChangeBandFilters()
     self.SetTxLevel()
+  def OnButtonAntenna(self, event):
+    btn = event.GetEventObject()
+    self.antenna_index = btn.index
+    self.ChangeBandFilters()
   def ChangeBandFilters(self):
+    self.SetControlBit(0x00, 13, self.antenna_index)
     highest = self.band
     freq = self.conf.BandEdge.get(highest, (0, 0))[0]
     for pane in self.application.multi_rx_screen.receiver_list:
@@ -374,31 +370,39 @@ class Hardware(BaseHardware):
       if freq < f:
         freq = f
         highest = pane.band
+    # Set Hermes Rx and Tx filters for C0 == 0x00, C2[7:1]
     Rx = self.conf.Hermes_BandDict.get(highest, 0)
-    self.SetControlByte(0, 2, Rx << 1)		# C0 index == 0, C2[7:1]: user output
+    self.SetControlByte(0, 2, Rx << 1, False)	# C0 index == 0, C2[7:1]: user output
     if self.conf.Hermes_BandDictEnTx:
       Tx = self.conf.Hermes_BandDictTx.get(self.band, 0)	# Use Tx filter
     else:
-      Tx = self.conf.Hermes_BandDict.get(self.band, 0)		# Use Rx filter
+      Tx = self.conf.Hermes_BandDict.get(self.band, 0)		# Use the Rx filter for the Tx band
     QS.set_hermes_filters(Rx, Tx)
-    self.ChangeAlexFilters()
+    if DEBUG: print("Change Hermes Band Filters: Antenna %d Rx 0x%X Tx 0x%X" % (self.antenna_index, Rx, Tx))
+    # Set Alex filters for C0 == 0x09
+    rx_value = Rx & 0x7F
+    if self.antenna_index:
+      rx_value |= 0x80
+    self.SetControlByte(0x09, 3, rx_value, False)	# C0 index == 0x09, C3, Rx filter
+    Tx = self.conf.Hermes_BandDictTx.get(self.band, 0)	# Tx filter
+    tx_value = Tx & 0x7F
+    if self.antenna_index:
+      tx_value |= 0x80
+    self.SetControlByte(0x09, 4, tx_value, False)	# C0 index == 0x09, C4, Tx filter
+    if DEBUG: print("Change Alex Band Filters: Antenna %d Rx 0x%X Tx 0x%X" % (self.antenna_index, rx_value, tx_value))
   def ChangeMode(self, mode):
     # mode is a string: "USB", "AM", etc.
     BaseHardware.ChangeMode(self, mode)
     self.mode = mode
-    if mode == "CWL" or mode == "CWU":
-      value = self.conf.keyupDelay
-    else:
-      value = 50
-    if value > 1023:
-      value = 1023
-    self.SetControlByte(0x10, 2, value & 0x3)		# cw_hang_time
-    self.SetControlByte(0x10, 1, (value >> 2) & 0xFF)	# cw_hang_time
-    if DEBUG: print ("Change keyup delay to", value)
     self.SetTxLevel()
   def OnSpot(self, level):
     # level is -1 for Spot button Off; else the Spot level 0 to 1000.
-    pass
+    if level >= 0 and self.conf.hermes_antenna_tuner == "Tune":
+      self.SetControlBit(0x09, 20, 1)
+      if DEBUG: print("OnSpot antenna tuner: On")
+    else:
+      self.SetControlBit(0x09, 20, 0)
+      if DEBUG: print("OnSpot antenna tuner: Off")
   def VarDecimGetChoices(self):		# return text labels for the control
     return self.var_rates
   def VarDecimGetLabel(self):		# return a text label for the control
@@ -453,7 +457,7 @@ class Hardware(BaseHardware):
   def ChangeTxLNA(self, value):		# LNA for Tx
     # value is -12 to +48
     value = ((value+12) & 0x3f) | 0x40 | 0x80
-    self.SetControlByte(0x0e, 3, value)		# C0 index == 0x0e, C3
+    self.SetControlByte(0x0e, 3, value, False)		# C0 index == 0x0e, C3
     QS.pc_to_hermes(self.pc2hermes)
     if DEBUG: print ("Change Tx LNA to", value)
   def SetTxLevel(self):
@@ -526,27 +530,82 @@ class Hardware(BaseHardware):
     else:
       QS.set_PTT(0)
     return start, stop	# Return actual frequencies after all phase rounding
-  def EnablePowerAmp(self, enable):
-    if enable:
-      self.pc2hermes[4 * 9 + 1] |= 0x08		# C0 index == 9, C2
+  def ImmediateChange(self, name):
+    if name == 'keyupDelay':
+      value = self.conf.keyupDelay
+      if value > 1023:
+        value = 1023
+      self.SetControlByte(0x10, 2, value & 0x3, False)		# cw_hang_time
+      self.SetControlByte(0x10, 1, (value >> 2) & 0xFF, False)	# cw_hang_time
+      if DEBUG: print ("Change keyup delay to", value)
+    elif name in ('hermes_tx_buffer_latency', 'hermes_PTT_hang_time'):
+      lat = self.conf.hermes_tx_buffer_latency
+      if lat < 0:
+        lat = 0
+      elif lat > 127:
+        lat = 127
+      hang = self.conf.hermes_PTT_hang_time
+      if hang < 0:
+        hang = 0
+      elif hang > 31:
+        hang = 31
+      self.pc2hermeslitewritequeue[0:5] = 0x17 | 0x40, 0, 0, hang, lat
+      self.WriteQueue(1)
+      if DEBUG: print ("Change tx_buffer_latency %d, PTT_hang_time %d" % (lat, hang))
+    elif name == 'hermes_PWM':
+      if self.conf.hermes_PWM[0:4] == 'Fan ':
+        self.SetControlBit(0x00, 11, 0)
+      else:
+        self.SetControlBit(0x00, 11, 1)
+      if DEBUG: print ("Change hermes_PWM to", self.conf.hermes_PWM)
+    elif name == 'hermes_disable_sync':
+      if self.conf.hermes_disable_sync:
+        self.SetControlBit(0x00, 12, 1)
+        if DEBUG: print ("Set hermes_disable_sync")
+      else:
+        self.SetControlBit(0x00, 12, 0)
+        if DEBUG: print ("Clear hermes_disable_sync")
+    elif name == 'hermes_disable_watchdog':
+      if self.conf.hermes_disable_watchdog:
+        self.pc2hermeslitewritequeue[0:5] = 0x39 | 0x40, 0x09, 0, 0, 0
+        self.WriteQueue(1)
+        if DEBUG: print ("Set hermes_disable_watchdog")
+      else:
+        self.pc2hermeslitewritequeue[0:5] = 0x39 | 0x40, 0x08, 0, 0, 0
+        self.WriteQueue(1)
+        if DEBUG: print ("Clear hermes_disable_watchdog")
+    elif name == 'hermes_reset_on_disconnect':
+      if self.conf.hermes_reset_on_disconnect:
+        self.pc2hermeslitewritequeue[0:5] = 0x3A | 0x40, 0, 0, 0, 0x01
+        self.WriteQueue(1)
+        if DEBUG: print ("Set hermes_reset_on_disconnect")
+      else:
+        self.pc2hermeslitewritequeue[0:5] = 0x3A | 0x40, 0, 0, 0, 0
+        self.WriteQueue(1)
+        if DEBUG: print ("Clear hermes_reset_on_disconnect")
+    elif name == 'hermes_lowpwr_tr_enable':
+      if self.conf.hermes_lowpwr_tr_enable:
+        self.SetControlBit(0x09, 18, 1)
+      else:
+        self.SetControlBit(0x09, 18, 0)
+      if DEBUG: print ("Change disable T/R in low power to", self.conf.hermes_lowpwr_tr_enable)
+    elif name == 'hermes_power_amp':
+      if self.conf.hermes_power_amp:
+        self.SetControlBit(0x09, 19, 1)
+      else:
+        self.SetControlBit(0x09, 19, 0)
+      if DEBUG: print ("Change power_amp to", self.conf.hermes_power_amp)
+    elif name == 'Hermes_BandDictEnTx':
+      if self.conf.Hermes_BandDictEnTx:
+        self.SetControlBit(0x09, 22, 1)
+      else:
+        self.SetControlBit(0x09, 22, 0)
+      if DEBUG: print ("Change Alex manual mode to", self.conf.Hermes_BandDictEnTx)
+      self.ChangeBandFilters()
+    elif name == 'hermes_antenna_tuner':
+      pass
     else:
-      self.pc2hermes[4 * 9 + 1] &= ~0x08
-    QS.pc_to_hermes(self.pc2hermes)
-    if DEBUG: print ("Change PwrAmp 0x%X" % (self.pc2hermes[4*9 + 1]))
-  def SetLowPwrEnable(self, enable):
-    if enable:
-      self.pc2hermes[4 * 9 + 1] |= 0x04		# C0 index == 9, C2
-    else:
-      self.pc2hermes[4 * 9 + 1] &= ~0x04
-    QS.pc_to_hermes(self.pc2hermes)
-    if DEBUG: print ("Change LpPwrEnable 0x%X" % (self.pc2hermes[4*9 + 1]))
-  def DisableSyncFreq(self, value):	# Thanks to Steve, KF7O
-    if value:
-      self.pc2hermes[4 * 0 + 2] |= 0x10   # C0 index == 0, C3
-    else:
-      self.pc2hermes[4 * 0 + 2] &= ~0x10
-    QS.pc_to_hermes(self.pc2hermes)
-    if DEBUG: print ("Change SyncFreq 0x%X" % (self.pc2hermes[4*0 + 2]))
+      if DEBUG: print ("Immediate change: no such name", name)
   def EnableBiasChange(self, enable):
     # Bias settings are in location 12, 13, 14, 15, and are not sent unless C1 == 0x06
     if enable:
@@ -607,7 +666,7 @@ class Hardware(BaseHardware):
       time.sleep(dt)
       dt *= 2
       if dt > 0.300:
-        print("ERROR: Hermes-Lite write queue timeout")
+        print("ERROR: Hermes-Lite write queue timeout, queue 0x%x 0x%x 0x%x 0x%x 0x%x" % tuple(self.pc2hermeslitewritequeue[0:5]))
         return
     ## Send next write(s)
     QS.pc_to_hermeslite_writequeue(self.pc2hermeslitewritequeue)
