@@ -14,7 +14,7 @@ static char port_name[QUISK_SC_SIZE];	// serial port name
 static PyObject * start_up(void);	// open the serial port
 static void shut_down(void);		// close the serial port
 static void modem_status(void);		// test modem bits
-#define MSG_SIZE	120
+#define MSG_SIZE	(50 + QUISK_SC_SIZE)
 
 //int quisk_serial_key_errors = 0;
 int quisk_serial_key_down;		// The cts or dsr bit for CW key is asserted
@@ -28,7 +28,8 @@ PyObject * quisk_open_key(PyObject * self, PyObject * args)
 
 	if (!PyArg_ParseTuple (args, "sss", &port, &cts, &dsr))
 		return NULL;
-	strncpy(port_name, port, QUISK_SC_SIZE);
+	strncpy(port_name, port, QUISK_SC_SIZE - 1);
+	port_name[QUISK_SC_SIZE - 1] = 0;
 	//quisk_serial_cts and dsr are "None", "CW", "PTT"; and "when high" or "when low"
 	use_cts = * cts;		// 'N', 'C', 'P'
 	reverse_cts = strstr(cts, "when low") != NULL;
@@ -148,6 +149,9 @@ static void modem_status(void)
 #else
 // Not MS Windows and not __MACH__:
 
+// Access the serial port.  This code sets DTR high, and monitors DSR and CTS.
+// When DSR is high set the RTS signal high. When DSR goes low set RTS low after a delay.
+
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <termios.h>
@@ -157,11 +161,30 @@ static int fdComm = -1;			// File descriptor to read the serial port
 static void modem_status(void)
 {
 	int bits;
+	struct timeval tv;
+	double time;
+	static double time0=0;	// time when the key was last down
 
         if (fdComm >= 0) {
 	        ioctl(fdComm, TIOCMGET, &bits);	// read modem bits
 	        bit_cts = bits & TIOCM_CTS;
 	        bit_dsr = bits & TIOCM_DSR;
+		if (bit_dsr) {
+			if ( ! (bits & TIOCM_RTS)) {	// set RTS
+				bits |= TIOCM_RTS;
+				ioctl(fdComm, TIOCMSET, &bits);
+			}
+			gettimeofday(&tv, NULL);
+			time0 = tv.tv_sec + tv.tv_usec / 1.0E6;		// time is in seconds
+		}
+		else if (bits & TIOCM_RTS) {	// clear RTS after a delay
+			gettimeofday(&tv, NULL);
+			time = tv.tv_sec + tv.tv_usec / 1.0E6;
+			if (time - time0 > pt_quisk_sound_state->quiskKeyupDelay * 1E-3) {
+				bits &= ~TIOCM_RTS;
+				ioctl(fdComm, TIOCMSET, &bits);
+			}
+		}
         }
 }
 
@@ -169,6 +192,7 @@ static PyObject * start_up(void)
 {
 	int bits;
 	char msg[MSG_SIZE];
+	struct timespec tspec;
 
 	fdComm = open(port_name, O_RDWR | O_NOCTTY);
 	if (fdComm < 0) {
@@ -179,10 +203,12 @@ static PyObject * start_up(void)
 	else {
 		ioctl(fdComm, TIOCMGET, &bits);		// read modem bits
 		bits |= TIOCM_DTR;			// Set DTR
-		bits &= ~TIOCM_RTS;			// Clear RTS
+		bits &= ~TIOCM_RTS;			// Clear RTS at first
 		ioctl(fdComm, TIOCMSET, &bits);
 	}
-	QuiskSleepMicrosec(10000);
+	tspec.tv_sec = 0;
+	tspec.tv_nsec = 10000 * 1000;
+	nanosleep(&tspec, NULL);
 	startup_error = 0;
 	return PyUnicode_FromString("");
 }
