@@ -4,6 +4,8 @@
 # This module is used to add extra widgets to the QUISK screen.
 
 from __future__ import print_function
+from __future__ import absolute_import
+from __future__ import division
 
 import wx, time
 import _quisk as QS
