@@ -25,15 +25,20 @@
 // 3 == Copy transmit audio to the output.
 #define TEST_AUDIO	0
 
+#define QUISK_CWKEY_DOWN	(quisk_hardware_cwkey || quisk_serial_key_down || quisk_midi_cwkey)
 
 #ifdef MS_WINDOWS
 #define QUISK_SHUT_RD	SD_RECEIVE
 #define QUISK_SHUT_BOTH	SD_BOTH
+#include <synchapi.h>
+extern CRITICAL_SECTION QuiskCriticalSection;
+extern PyObject * QuiskPrintf(char *, ...);
 #else
 #define SOCKET  int
 #define INVALID_SOCKET	-1
 #define QUISK_SHUT_RD	SHUT_RD
 #define QUISK_SHUT_BOTH	SHUT_RDWR
+#define QuiskPrintf     printf
 #endif
 
 #if PY_MAJOR_VERSION >= 3
@@ -67,12 +72,14 @@ typedef enum _rx_mode {
  * \brief Specifies which driver a \c sound_dev is opened with
  */
 
-typedef enum {
-	Int16,
-	Int24,
+typedef enum {  // In order of preference.
 	Int32,
-	Float32
+	Int16,
+	Float32,
+	Int24
 } sound_format_t;
+// Keep this array of names up to date.
+extern char * sound_format_names[4];     // = {"Int32", "Int16", "Float32", "Int24"} ;
 
 typedef enum {
 	t_Capture,
@@ -91,12 +98,15 @@ typedef enum dev_driver{
    DEV_DRIVER_ALSA,
    DEV_DRIVER_PULSEAUDIO,
    DEV_DRIVER_DIRECTX,
-   DEV_DRIVER_SOUNDIO
+   DEV_DRIVER_WASAPI,
+   DEV_DRIVER_WASAPI2,
 } dev_driver_t;
 
 typedef enum {
+	SHUTDOWN,
 	STARTING,
 	RECEIVE,
+// All states greater than RECEIVE are assumed to be transmit states.
 	HARDWARE_CWKEY,
 	HARDWARE_PTT,
 	SOFTWARE_CWKEY,
@@ -129,10 +139,11 @@ struct sound_dev {				// data for sound capture or playback device
 	int read_frames;
 	int latency_frames;			// desired latency in audio play samples
 	int play_buf_size;			// size of the sound card playback buffer in frames or bytes
+        int old_key;                            // previous key up/down state
 	int use_float;				// DirectX: Use IEEE floating point
 	int dataPos;				// DirectX: data position
-	int oldPlayPos;				// DirectX: previous value of playPos
 	int play_delay;				// DirectX: bytes of sound available to play
+	int old_play_delay;			// DirectX: previous bytes of sound available to play
 	int started;				// DirectX: started flag or state
 	int dev_error;				// read or write error
 	int dev_underrun;			// lack of samples to play
@@ -154,8 +165,19 @@ struct sound_dev {				// data for sound capture or playback device
 	sound_format_t sound_format;	// format of sound array for the sound device
 	device_index_t dev_index;	// identify devices
 	void * device_data;		// special data for each sound device
+        double TimerTime0;              // Used to print debug messages
+	// Variables used to correct differences in sample rates:
+	int cr_correction;
+	int cr_delay;
+	double cr_average_fill;
+	int cr_average_count;
+	int cr_sample_time;
+	int cr_correct_time;
 } ;
 
+extern struct sound_dev quisk_Playback;
+extern struct sound_dev quisk_MicPlayback;
+
 struct sound_conf {
 	char dev_capt_name[QUISK_SC_SIZE];
 	char dev_play_name[QUISK_SC_SIZE];
@@ -190,6 +212,7 @@ struct sound_conf {
 	char IQ_server[IP_SIZE];	//IP address of optional streaming IQ server (pulseaudio)
 	int verbose_pulse;      // verbose output for pulse audio
 	int verbose_sound;	// verbose output for other sound systems
+	int quiskKeyupDelay;	// keup delay in milliseconds
 } ;
 
 enum quisk_rec_state {
@@ -229,7 +252,7 @@ void QuiskWavWriteF(struct QuiskWav *, float *, int);
 int QuiskWavReadOpen(struct QuiskWav *, char *, short, short, short, int, double);
 void QuiskWavReadC(struct QuiskWav *, complex double *, int);
 void QuiskWavReadD(struct QuiskWav *, double *, int);
-void QuiskMeasureRate(const char *, int, int);
+void QuiskMeasureRate(const char *, int, int, int);
 void quisk_record_audio(struct wav_file *, complex double *, int);
 
 extern struct sound_conf quisk_sound_state, * pt_quisk_sound_state;
@@ -245,7 +268,6 @@ extern double quisk_mic_preemphasis;	// Mic preemphasis 0.0 to 1.0; or -1.0
 extern double quisk_mic_clip;			// Mic clipping; try 3.0 or 4.0
 extern int quisk_noise_blanker;			// Noise blanker level, 0 for off
 extern int quisk_sidetoneCtrl;			// sidetone control value 0 to 1000
-extern int quiskKeyupDelay;			// key-up delay from the config file
 extern double quisk_audioVolume;		// volume control for radio sound playback, 0.0 to 1.0
 extern int quiskImdLevel;				// level for rxMode IMD
 extern int quiskTxHoldState;			// state machine for Tx wait for repeater frequency shift
@@ -265,6 +287,7 @@ extern int quisk_serial_key_errors;		// Error count for the Quisk internal seria
 extern double quisk_sidetoneVolume;		// Audio output level of the CW sidetone, 0.0 to 1.0
 extern int quisk_serial_key_down;		// The state of the serial port CW key
 extern int quisk_hardware_cwkey;		// The state of the hardware CW key from UDP or USB
+extern int quisk_midi_cwkey;			// The state of the MIDI CW key
 extern int quisk_sidetoneFreq;			// Frequency in hertz for the sidetone
 extern int quisk_active_sidetone;		// Whether and how to generate a sidetone
 extern int quisk_isFDX;				// Are we in full duplex mode?
@@ -281,9 +304,12 @@ extern PyObject * quisk_set_ampl_phase(PyObject * , PyObject *);
 extern PyObject * quisk_capt_channels(PyObject * , PyObject *);
 extern PyObject * quisk_play_channels(PyObject * , PyObject *);
 extern PyObject * quisk_micplay_channels(PyObject * , PyObject *);
-extern PyObject * quisk_sound_devices(PyObject * , PyObject *);
-extern PyObject * quisk_pa_sound_devices(PyObject * , PyObject *);
-extern PyObject * quisk_sio_sound_devices(PyObject * , PyObject *);
+extern PyObject * quisk_alsa_sound_devices(PyObject * , PyObject *);
+extern PyObject * quisk_directx_sound_devices(PyObject * , PyObject *);
+extern PyObject * quisk_portaudio_sound_devices(PyObject * , PyObject *);
+extern PyObject * quisk_pulseaudio_sound_devices(PyObject * , PyObject *);
+extern PyObject * quisk_wasapi_sound_devices(PyObject * , PyObject *);
+extern PyObject * quisk_dummy_sound_devices(PyObject * , PyObject *);
 extern PyObject * quisk_sound_errors(PyObject *, PyObject *);
 extern PyObject * quisk_set_file_record(PyObject *, PyObject *);
 extern PyObject * quisk_set_file_name(PyObject *, PyObject *, PyObject *);
@@ -305,6 +331,8 @@ extern PyObject * quisk_freedv_set_squelch_en(PyObject *, PyObject *);
 extern PyObject * quisk_open_key(PyObject *, PyObject *);
 extern PyObject * quisk_close_key(PyObject *, PyObject *);
 extern PyObject * quisk_set_sound_name(PyObject *, PyObject *);
+extern PyObject * quisk_alsa_control_midi(PyObject *, PyObject *, PyObject *);
+extern PyObject * quisk_wasapi_control_midi(PyObject *, PyObject *, PyObject *);
 
 // These function pointers are the Start/Stop/Read interface for
 // the SDR-IQ and any other C-language extension modules that return
@@ -322,7 +350,7 @@ void quisk_play_samples(complex double *, int);
 void quisk_play_zeros(int);
 void quisk_start_sound(void);
 int quisk_get_overrange(void);
-void quisk_mixer_set(char *, int, PyObject *, char *, int);
+void quisk_alsa_mixer_set(char *, int, PyObject *, char *, int);
 int quisk_read_sound(void);
 int quisk_process_microphone(int, complex double *, int);
 void quisk_open_mic(void);
@@ -342,9 +370,10 @@ void quisk_check_freedv_mode(void);
 void quisk_calc_audio_graph(double, complex double *, double *, int, int);
 int QuiskDeltaMsec(int);
 void * quisk_make_sidetone(struct sound_dev *, int);
-void quisk_set_play_state(int);
+void * quisk_make_txIQ(struct sound_dev *, int);
+int quisk_play_sidetone(struct sound_dev *);
+void quisk_set_play_state(void);
 void quisk_poll_hardware_key(void);
-void quisk_set_sidetone(void);
 
 // Functions supporting digital voice codecs
 typedef int  (* ty_dvoice_codec_rx)(complex double *, double *, int, int);
@@ -373,10 +402,16 @@ void quisk_close_sound_pulseaudio(void);
 void quisk_cork_pulseaudio(struct sound_dev *, int);
 void quisk_flush_pulseaudio(struct sound_dev *);
 
-int  quisk_read_soundio(struct sound_dev *, complex double *);
-void quisk_write_soundio(struct sound_dev *, int, complex double *);
-void quisk_start_sound_soundio(struct sound_dev **, struct sound_dev **);
-void quisk_close_sound_soundio(struct sound_dev **, struct sound_dev **);
+int  quisk_read_directx(struct sound_dev *, complex double *);
+void quisk_play_directx(struct sound_dev *, int, complex double *, int, double);
+void quisk_start_sound_directx(struct sound_dev **, struct sound_dev **);
+void quisk_close_sound_directx(struct sound_dev **, struct sound_dev **);
+
+int  quisk_read_wasapi(struct sound_dev *, complex double *);
+void quisk_write_wasapi(struct sound_dev *, int, complex double *, double);
+void quisk_play_wasapi(struct sound_dev *, int, complex double *, double);
+void quisk_start_sound_wasapi(struct sound_dev **, struct sound_dev **);
+void quisk_close_sound_wasapi(struct sound_dev **, struct sound_dev **);
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 /*
@@ -400,9 +435,10 @@ int import_quisk_api(void);	// used to initialize Quisk_API
 #define QuiskSleepMicrosec	(*(	void	(*)	(int)			)Quisk_API[5])
 #define QuiskPrintTime		(*(	void	(*)	(const char *, int)	)Quisk_API[6])
 #define quisk_sample_source	(*(	void	(*)	(ty_sample_start, ty_sample_stop, ty_sample_read)	)Quisk_API[7])
-#define quisk_dvoice_freedv	(*(	void	(*)	(ty_dvoice_codec_rx, ty_dvoice_codec_tx)	)Quisk_API[8])
-#define quisk_is_key_down	(*(	int	(*)	(void)			)Quisk_API[9])
+#define quisk_dvoice_freedv	(*(	void	(*)	(ty_dvoice_codec_rx, ty_dvoice_codec_tx)        	)Quisk_API[8])
+#define quisk_is_key_down	(*(	int	(*)	(void)			                                )Quisk_API[9])
 #define quisk_sample_source4	(*(	void	(*)	(ty_sample_start, ty_sample_stop, ty_sample_read, ty_sample_write)	)Quisk_API[10])
+#define strMcpy                 (*(     char *  (*)     (char *, const char *, size_t)                          )Quisk_API[11])
 
 #else
 // Used to export symbols from _quisk in quisk.c
@@ -417,11 +453,12 @@ void	quisk_sample_source(ty_sample_start, ty_sample_stop, ty_sample_read);
 void	quisk_dvoice_freedv(ty_dvoice_codec_rx, ty_dvoice_codec_tx);
 int	quisk_is_key_down(void);
 void	quisk_sample_source4(ty_sample_start, ty_sample_stop, ty_sample_read, ty_sample_write);
+char *  strMcpy(char *, const char *, size_t);
 
 #define QUISK_API_INIT	{ \
  &quisk_sound_state, &QuiskGetConfigInt, &QuiskGetConfigDouble, &QuiskGetConfigString, &QuiskTimeSec, \
  &QuiskSleepMicrosec, &QuiskPrintTime, &quisk_sample_source, &quisk_dvoice_freedv, &quisk_is_key_down, \
- &quisk_sample_source4 \
+ &quisk_sample_source4, &strMcpy \
  }
 
 #endif
