@@ -1124,7 +1124,7 @@ class App(wx.App):
     QS.open_sound(0, conf.data_poll_usec, conf.latency_millisecs,
                 conf.tx_ip, conf.tx_audio_port,
                 conf.mic_sample_rate, conf.mic_channel_I, conf.mic_channel_Q,
-				conf.mic_out_volume, conf.mic_playback_rate)
+				0.7, conf.mic_playback_rate)
     self.Bind(wx.EVT_IDLE, self.graph.OnIdle)
     frame.Show()
     self.NewFreq(1000000, 30000000)
