@@ -296,21 +296,29 @@ class SliderBoxV(wx.BoxSizer):
     if display:		# Display the slider value when it is thumb'd
       self.text_ctrl = wx.StaticText(parent, -1, str(themax))
       self.text_ctrl.SetFont(button_font)
+      charw = self.text_ctrl.GetCharWidth()
       w1, self.text_height = self.text_ctrl.GetSize()	# Measure size with max number
+      w1 += charw
       self.text_ctrl.SetLabel(str(themin))
       w3, h3 = self.text_ctrl.GetSize()	# Measure size with min number
+      w3 += charw
       self.text_ctrl.SetLabel(text)
       w2, h2 = self.text_ctrl.GetSize()	# Measure size with text
-      self.width = max(w1, w2, w3, sw) + self.text_ctrl.GetCharWidth()
+      w2 += charw
+      self.width = max(w1, w2, w3, sw)
       self.text_ctrl.SetSizeHints(self.width, -1, self.width)
+      self.slider.SetSizeHints(self.width, -1, self.width)
       self.slider.Bind(wx.EVT_SCROLL_THUMBTRACK, self.Change)
       self.slider.Bind(wx.EVT_SCROLL_THUMBRELEASE, self.ChangeDone)
     else:
       self.text_ctrl = wx.StaticText(parent, -1, text)
       self.text_ctrl.SetFont(button_font)
+      charw = self.text_ctrl.GetCharWidth()
       w2, self.text_height = self.text_ctrl.GetSize()	# Measure size with text
-      self.width = max(w2, sw) + self.text_ctrl.GetCharWidth()
+      w2 += charw
+      self.width = max(w2, sw)
       self.text_ctrl.SetSizeHints(self.width, -1, self.width)
+      self.slider.SetSizeHints(self.width, -1, self.width)
     self.text_ctrl.SetForegroundColour(parent.GetForegroundColour())
     self.Add(self.text_ctrl, 0, 0)
     self.Add(self.slider, 1, 0)
@@ -576,9 +584,9 @@ class QuiskRepeatbutton(QuiskButtons, wx.lib.buttons.GenButton):
     wx.lib.buttons.GenButton.OnLeftDown(self, event)
   def OnLeftUp(self, event):
     if self.IsEnabled():
-      self.SendCommand(self.up_command)
       self.repeat_state = 0
       self.timer.Stop()
+      self.SendCommand(self.up_command)
     wx.lib.buttons.GenButton.OnLeftUp(self, event)
   def OnRightDown(self, event):
     if self.IsEnabled():
@@ -1221,7 +1229,7 @@ class RadioBtnPopup:
     self.first_button.decoration = u'\u21D2'
     self.second_button = QuiskBitmapButton(parent, self.OnSecondButton, _bitmap_menupop, use_right=True)
     self.pop_control.Add(self.first_button, 1, flag=wx.ALIGN_CENTER_VERTICAL)
-    self.pop_control.Add(self.second_button, 0, flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL|wx.LEFT, border=2)
+    self.pop_control.Add(self.second_button, 0, flag=wx.ALIGN_CENTER_VERTICAL|wx.LEFT, border=2)
     self.pop_control.Show(self.second_button, False)
     self.adjust = None
     self.first_button.index = 0
