@@ -5,7 +5,7 @@ from __future__ import print_function
 from __future__ import absolute_import
 from __future__ import division
 
-import sys, struct, threading, time, traceback, math
+import sys, struct, time, traceback, math
 from quisk_hardware_model import Hardware as BaseHardware
 import _quisk as QS
 
@@ -58,6 +58,8 @@ class Hardware(BaseHardware):
     BaseHardware.__init__(self, app, conf)
     self.usb_dev = None
     self.vfo = None
+    self.mode = None
+    self.band = None
     self.repeater_freq = None		# original repeater output frequency
     try:
       self.repeater_delay = conf.repeater_delay		# delay for changing repeater frequency in seconds
@@ -65,7 +67,6 @@ class Hardware(BaseHardware):
       self.repeater_delay = 0.25
     self.repeater_time0 = 0			# time of repeater change in frequency
     self.ptt_button = 0
-    self.key_thread = None
     self.si570_i2c_address = conf.si570_i2c_address
   def open(self):			# Called once to open the Hardware
     global usb
@@ -111,9 +112,6 @@ class Hardware(BaseHardware):
         if len(sound) > 50:
           sound = sound[0:30] + '|||' + sound[-17:]
         text = 'Capture from SoftRock USB on %s, Firmware %s' % (sound, ver)
-        if self.conf.name_of_mic_play and self.conf.key_poll_msec:
-          self.key_thread = KeyThread(usb_dev, self.conf.key_poll_msec / 1000.0)
-          self.key_thread.start()
     #self.application.bottom_widgets.info_text.SetLabel(text)
     if DEBUG and usb_dev:
       print ('Startup freq', self.GetStartupFreq())
@@ -124,9 +122,7 @@ class Hardware(BaseHardware):
       print ('Smooth tune', sm)
     return text
   def close(self):			# Called once to close the Hardware
-    if self.key_thread:
-      self.key_thread.stop()
-      self.key_thread = None
+    pass
   def ChangeFrequency(self, tune, vfo, source='', band='', event=None):
     if self.usb_dev and self.vfo != vfo:
       if self.conf.si570_direct_control:
@@ -138,6 +134,30 @@ class Hardware(BaseHardware):
         print ('Change to', vfo)
         print ('Run freq', self.GetFreq())
     return tune, vfo
+  def ChangeBand(self, band):
+    # band is a string: "60", "40", "WWV", etc.
+    BaseHardware.ChangeBand(self, band)
+    self.band = band
+    self.SetTxLevel()
+  def ChangeMode(self, mode):
+    # mode is a string: "USB", "AM", etc.
+    BaseHardware.ChangeMode(self, mode)
+    self.mode = mode
+    QS.set_cwkey(0)
+    self.SetTxLevel()
+  def SetTxLevel(self):
+    tx_level = self.conf.tx_level.get(self.band, 70)
+    if self.mode[0:3] in ('DGT', 'FDV'):			# Digital modes; change power by a percentage
+      reduc = self.application.digital_tx_level
+    else:
+      reduc = self.application.tx_level
+    tx_level = int(tx_level * reduc / 100.0 + 0.5)  
+    if tx_level < 0:
+      tx_level = 0
+    elif tx_level > 100:
+      tx_level = 100
+    QS.set_mic_out_volume(tx_level)
+    if DEBUG: print("Change tx_level to", tx_level)
   def ReturnFrequency(self):
     # Return the current tuning and VFO frequency.  If neither have changed,
     # you can return (None, None).  This is called at about 10 Hz by the main.
@@ -250,30 +270,19 @@ class Hardware(BaseHardware):
                                    ((rfreq_int & 0xf) << 28) + rfreq_frac)
     self.usb_dev.ctrl_transfer(OUT, 0x30, self.si570_i2c_address + 0x700, 0, s)
     return True		# Success
-
-class KeyThread(threading.Thread):
-  """Create a thread to monitor the key state."""
-  def __init__(self, dev, poll_secs):
-    self.usb_dev = dev
-    self.poll_secs = poll_secs
-    threading.Thread.__init__(self)
-    self.doQuit = threading.Event()
-    self.doQuit.clear()
-  def run(self):
-    while not self.doQuit.isSet():
-      try:		# Test key up/down state
-        ret = self.usb_dev.ctrl_transfer(IN, 0x51, 0, 0, 1)
-      except usb.core.USBError:
+  def PollCwKey(self):  # Called frequently by Quisk to check the CW key status
+    if not self.usb_dev:
+      return
+    if self.mode not in ('CWU', 'CWL'):
+      return
+    try:		# Test key up/down state
+      ret = self.usb_dev.ctrl_transfer(IN, 0x51, 0, 0, 1)
+    except:
+      QS.set_cwkey(0)
+      if DEBUG: traceback.print_exc()
+    else:
+      # bit 0x20 is the tip, bit 0x02 is the ring (ring not used)
+      if ret[0] & 0x20 == 0:		# Tip: key is down
+        QS.set_cwkey(1)
+      else:			# key is up
         QS.set_cwkey(0)
-        if DEBUG: traceback.print_exc()
-      else:
-        # bit 0x20 is the tip, bit 0x02 is the ring (ring not used)
-        if ret[0] & 0x20 == 0:		# Tip: key is down
-          QS.set_cwkey(1)
-        else:			# key is up
-          QS.set_cwkey(0)
-      time.sleep(self.poll_secs)
-    QS.set_cwkey(0)
-  def stop(self):
-    """Set a flag to indicate that the thread should end."""
-    self.doQuit.set()
