@@ -1,6 +1,8 @@
 /*
  * This module provides sound access for QUISK using the portaudio library.
 */
+#ifdef QUISK_HAVE_PORTAUDIO
+
 #include <Python.h>
 #include <complex.h>
 #include <math.h>
@@ -66,8 +68,8 @@ int quisk_read_portaudio(struct sound_dev * dev, complex double * cSamples)
 void quisk_play_portaudio(struct sound_dev * playdev, int nSamples, complex double * cSamples,
 		int report_latency, double volume)
 {	// play the samples; write them to the portaudio soundcard
-	int i, n, index;
-	long delay;
+	int i, n;
+	long delay, write_available;
 	float fi, fq;
 	PaError error;
 
@@ -75,58 +77,60 @@ void quisk_play_portaudio(struct sound_dev * playdev, int nSamples, complex doub
 		return;
 
 	// "delay" is the number of samples left in the play buffer
-	delay = playdev->play_buf_size - Pa_GetStreamWriteAvailable(playdev->handle);
-	//printf ("play available %ld\n", Pa_GetStreamWriteAvailable(playdev->handle));
+	write_available = Pa_GetStreamWriteAvailable(playdev->handle);
+	delay = playdev->play_buf_size - write_available;
+	playdev->cr_average_fill += (double)(delay + nSamples / 2) / (playdev->latency_frames * 2);
+	playdev->cr_average_count++;
 	playdev->dev_latency = delay;
 	if (report_latency) {		// Report for main playback device
 		quisk_sound_state.latencyPlay = delay;
 	}
-//printf ("nSamples %d, delay %ld\n", nSamples, delay);
-	index = 0;
-#if 0
-	// Timing is too crude to support this logic
-	if (nSamples + delay > playdev->latency_frames * 9 / 10) {
-		nSamples--;
-#if DEBUG_IO
-		printf("Remove a sample %s nSamples %4d  delay %4d  total %4d\n", playdev->name, nSamples, (int)delay, nSamples + (int)delay);
-#endif
+	switch(playdev->started) {
+	case 0:	 // Starting state
+		playdev->started = 1;
+		nSamples = playdev->latency_frames - delay;
+		for (i = 0; i < nSamples; i++)
+			cSamples[i] = 0;
+		break;
+	case 1:	// Normal run state
+		// Check for underrun
+		if (delay < 16) {		// Buffer is too empty; fill it back up with zeros.
+			if (quisk_sound_state.verbose_sound)
+				printf("Underrun for %s\n", playdev->stream_description);
+			nSamples = playdev->latency_frames;
+			for (i = 0; i < nSamples; i++)
+				cSamples[i] = 0;
+		}
+		// Check if play buffer is too full.
+		else if (nSamples > write_available) {
+			quisk_sound_state.write_error++;
+			playdev->dev_error++;
+			if (quisk_sound_state.verbose_sound)
+				QuiskPrintf ("Buffer too full for %s\n", playdev->stream_description);
+			nSamples = 0;
+			playdev->started = 2;
+		}
+		break;
+	case 2:	 // Buffer is too full; wait for it to drain
+		if (delay < playdev->latency_frames) {
+			playdev->started = 1;
+			if (quisk_sound_state.verbose_sound)
+				QuiskPrintf("Resume adding samples for %s\n", playdev->stream_description);
+		}
+		else {
+			nSamples = 0;
+		}
+		break;
 	}
-	else if(nSamples + delay < playdev->latency_frames * 5 / 10) {
-		cSamples[nSamples] = cSamples[nSamples - 1];
-		nSamples++;
-#if DEBUG_IO
-		printf("Add    a sample %s nSamples %4d  delay %4d  total %4d\n", playdev->name, nSamples, (int)delay, nSamples + (int)delay);
-#endif
-	}
-#endif
-	if (nSamples + delay > playdev->latency_frames) {		// too many samples
-		index = nSamples + delay - playdev->latency_frames;	// write only the most recent samples
-		if (index > nSamples)
-			index = nSamples;
-		quisk_sound_state.write_error++;
-		playdev->dev_error++;
-#if DEBUG_IO
-		printf("Discard %d of %d samples at %d delay\n", index, nSamples, (int)delay);
-#endif
-		if (nSamples == index)		// no samples to play
-			return;
-	}
-	else if (delay < 16) {		// Buffer is too empty; fill it back up with zeros.
-		n = playdev->latency_frames * 7 / 10 - nSamples;
-#if DEBUG_IO
-		printf("Add %d zero samples at %ld delay\n", n, delay);
-#endif
-		for (i = 0; i < n; i++)
-			cSamples[nSamples++] = 0;
-	}
-	for (i = 0, n = index; n < nSamples; i += playdev->num_channels, n++) {
+	if (nSamples <= 0)
+		return;
+	for (i = 0, n = 0; n < nSamples; i += playdev->num_channels, n++) {
 		fi = volume * creal(cSamples[n]);
 		fq = volume * cimag(cSamples[n]);
 		fbuffer[i + playdev->channel_I] = fi / CLIP32;
 		fbuffer[i + playdev->channel_Q] = fq / CLIP32;
 	}
-	error = Pa_WriteStream ((PaStream * )playdev->handle, fbuffer, nSamples - index);
-//printf ("Write %d\n", nSamples - index);
+	error = Pa_WriteStream ((PaStream * )playdev->handle, fbuffer, nSamples);
 	if (error == paNoError)
 		;
 	else if (error == paOutputUnderflowed) {
@@ -137,7 +141,7 @@ void quisk_play_portaudio(struct sound_dev * playdev, int nSamples, complex doub
 		quisk_sound_state.write_error++;
 		playdev->dev_error++;
 #if DEBUG_IO
-		printf ("Play error: %s\n", Pa_GetErrorText(error));
+		printf ("Portaudio play error: %s\n", Pa_GetErrorText(error));
 #endif
 	}
 }
@@ -218,7 +222,7 @@ static int quisk_pa_name2index (struct sound_dev * dev, int is_capture)
 			dev->portaudio_index = Pa_GetDefaultInputDevice();
 		else
 			dev->portaudio_index = Pa_GetDefaultOutputDevice();
-		strncpy (dev->msg1, "Using default portaudio device", QUISK_SC_SIZE);
+		strMcpy (dev->msg1, "Using default portaudio device", QUISK_SC_SIZE);
 		return 0;
 	}
 	if ( ! strncmp (dev->name, "portaudio#", 10)) {		// Integer index follows "#"
@@ -230,7 +234,10 @@ static int quisk_pa_name2index (struct sound_dev * dev, int is_capture)
 		}
 		else {
 			snprintf (quisk_sound_state.err_msg, QUISK_SC_SIZE,
-				"Can not find portaudio device number %s", dev->name + 10);
+				"Error: Can not find portaudio device number %s", dev->name + 10);
+			strMcpy(dev->dev_errmsg, quisk_sound_state.err_msg, QUISK_SC_SIZE);
+			if (quisk_sound_state.verbose_sound)
+				printf("%s\n", quisk_sound_state.err_msg);
 		}
 		return 1;
 	}
@@ -247,13 +254,19 @@ static int quisk_pa_name2index (struct sound_dev * dev, int is_capture)
 		}
 		if (dev->portaudio_index == -1)	{	// Error
 			snprintf (quisk_sound_state.err_msg, QUISK_SC_SIZE,
-				"Can not find portaudio device named %s", dev->name + 10);
+				"Error: Can not find portaudio device named %s", dev->name + 10);
+			strMcpy(dev->dev_errmsg, quisk_sound_state.err_msg, QUISK_SC_SIZE);
+			if (quisk_sound_state.verbose_sound)
+				printf("%s\n", quisk_sound_state.err_msg);
 			return 1;
 		}
 		return 0;
 	}
 	snprintf (quisk_sound_state.err_msg, QUISK_SC_SIZE,
-		"Did not recognize portaudio device %.90s", dev->name);
+		"Error: Did not recognize portaudio device %.90s", dev->name);
+	strMcpy(dev->dev_errmsg, quisk_sound_state.err_msg, QUISK_SC_SIZE);
+	if (quisk_sound_state.verbose_sound)
+		printf("%s\n", quisk_sound_state.err_msg);
 	return 1;
 }
 
@@ -267,7 +280,10 @@ static int quisk_open_portaudio (struct sound_dev * cDev, struct sound_dev * pDe
 	info_portaudio (cDev, pDev);
 
 	if (pDev && cDev && pDev->sample_rate != cDev->sample_rate) {
-		strncpy(quisk_sound_state.err_msg, "Capture and Play sample rates must be equal.", QUISK_SC_SIZE);
+		strMcpy(quisk_sound_state.err_msg, "Capture and Play sample rates must be equal.", QUISK_SC_SIZE);
+		strMcpy(cDev->dev_errmsg, quisk_sound_state.err_msg, QUISK_SC_SIZE);
+		if (quisk_sound_state.verbose_sound)
+			printf("%s\n", quisk_sound_state.err_msg);
 		return 1;
 	}
 
@@ -294,16 +310,35 @@ static int quisk_open_portaudio (struct sound_dev * cDev, struct sound_dev * pDe
 		error = Pa_OpenStream (&hndl, &cParams, &pParams,
 				(double)cDev->sample_rate, cDev->read_frames, 0, NULL, NULL);
 		pDev->handle = cDev->handle = (void *)hndl;
+		if (error != paNoError) {
+			strMcpy(quisk_sound_state.err_msg, Pa_GetErrorText(error), QUISK_SC_SIZE);
+			strMcpy(cDev->dev_errmsg, quisk_sound_state.err_msg, QUISK_SC_SIZE);
+			strMcpy(pDev->dev_errmsg, quisk_sound_state.err_msg, QUISK_SC_SIZE);
+			if (quisk_sound_state.verbose_sound)
+				printf("%s\n", quisk_sound_state.err_msg);
+		}
 	}
 	else if (cDev) {
 		error = Pa_OpenStream (&hndl, &cParams, NULL,
 				(double)cDev->sample_rate, cDev->read_frames, 0, NULL, NULL);
 		cDev->handle = (void *)hndl;
+		if (error != paNoError) {
+			strMcpy(quisk_sound_state.err_msg, Pa_GetErrorText(error), QUISK_SC_SIZE);
+			strMcpy(cDev->dev_errmsg, quisk_sound_state.err_msg, QUISK_SC_SIZE);
+			if (quisk_sound_state.verbose_sound)
+				printf("%s\n", quisk_sound_state.err_msg);
+		}
 	}
 	else if (pDev) {
 		error = Pa_OpenStream (&hndl, NULL, &pParams,
 				(double)pDev->sample_rate, 0, 0, NULL, NULL);
 		pDev->handle = (void *)hndl;
+		if (error != paNoError) {
+			strMcpy(quisk_sound_state.err_msg, Pa_GetErrorText(error), QUISK_SC_SIZE);
+			strMcpy(pDev->dev_errmsg, quisk_sound_state.err_msg, QUISK_SC_SIZE);
+			if (quisk_sound_state.verbose_sound)
+				printf("%s\n", quisk_sound_state.err_msg);
+		}
 	}
 	else {
 		error = paNoError;
@@ -318,15 +353,13 @@ static int quisk_open_portaudio (struct sound_dev * cDev, struct sound_dev * pDe
 			pDev->latency_frames = pDev->play_buf_size;
 		}
 	}
-#if DEBUG_IO
-	if (pDev) {
-		printf ("play_buf_size %d\n", pDev->play_buf_size);
-		printf ("latency_frames %d\n", pDev->latency_frames);
+	if (pDev && quisk_sound_state.verbose_sound) {
+		printf ("%s: portaudio play_buf_size %d\n", pDev->stream_description, pDev->play_buf_size);
+		printf ("%s: portaudio latency_frames %d\n", pDev->stream_description, pDev->latency_frames);
+		printf ("%s: portaudio suggested latency %lf\n", pDev->stream_description, pDev->portaudio_latency);
 	}
-#endif
 	if (error == paNoError)
 		return 0;
-	strncpy(quisk_sound_state.err_msg, Pa_GetErrorText(error), QUISK_SC_SIZE);
 	return 1;
 }
 
@@ -372,7 +405,7 @@ void quisk_start_sound_portaudio(struct sound_dev ** pCapture, struct sound_dev
 		}
 		pCapt++;
 	}
-	strncpy (quisk_sound_state.msg1, (*pCapture)->msg1, QUISK_SC_SIZE);	// Primary capture device
+	strMcpy (quisk_sound_state.msg1, (*pCapture)->msg1, QUISK_SC_SIZE);	// Primary capture device
 	// Open remaining portaudio devices
 	pPlay = pPlayback;
 	while (*pPlay) {
@@ -387,7 +420,7 @@ void quisk_start_sound_portaudio(struct sound_dev ** pCapture, struct sound_dev
 		pPlay++;
 	}
      	if ( ! quisk_sound_state.msg1[0])	// Primary playback device
-		strncpy (quisk_sound_state.msg1, (*pPlayback)->msg1, QUISK_SC_SIZE);
+		strMcpy (quisk_sound_state.msg1, (*pPlayback)->msg1, QUISK_SC_SIZE);
 	pCapt = pCapture;
 	while (*pCapt) {
 		if ((*pCapt)->handle)
@@ -407,9 +440,6 @@ void quisk_close_sound_portaudio(void)
 	Pa_Terminate();
 }
 
-// Changes for MacOS support (__MACH__) thanks to Mario, DL3LSM.
-#if defined(__MACH__)
-
 static int device_list(PyObject * py, int input)
 {
 	int retNum = 0;
@@ -442,8 +472,8 @@ static int device_list(PyObject * py, int input)
 	
 }
 					   
-PyObject * quisk_sound_devices(PyObject * self, PyObject * args)
-{	// Return a list of ALSA device names [pycapt, pyplay]
+PyObject * quisk_portaudio_sound_devices(PyObject * self, PyObject * args)
+{	// Return a list of portaudio device names [pycapt, pyplay]
 	PyObject * pylist, * pycapt, * pyplay;
 	
 	if (!PyArg_ParseTuple (args, ""))
@@ -458,61 +488,47 @@ PyObject * quisk_sound_devices(PyObject * self, PyObject * args)
 	device_list(pyplay, 0);
 	return pylist;
 }
+#else		// No portaudio support
+#include <Python.h>
+#include <complex.h>
+#include "quisk.h"
 
-void quisk_play_alsa(struct sound_dev * dev, int nSamples,
-					 complex double * cSamples, int report_latency, double volume)
-{
+PyObject * quisk_portaudio_sound_devices(PyObject * self, PyObject * args)
+{	// Return a list of portaudio device names [pycapt, pyplay]
+	return quisk_dummy_sound_devices(self, args);
 }
 
-void quisk_start_sound_alsa(struct sound_dev ** pCapture, struct sound_dev ** pPlayback)
+void quisk_start_sound_portaudio(struct sound_dev ** pCapture, struct sound_dev ** pPlayback)
 {
+	struct sound_dev * pDev;
+	const char * msg = "No driver support for this device";
+
+	while (*pCapture) {
+		pDev = *pCapture++;
+		if (pDev->driver == DEV_DRIVER_PORTAUDIO) {
+			strMcpy(pDev->dev_errmsg, msg, QUISK_SC_SIZE);
+			if (quisk_sound_state.verbose_sound)
+				QuiskPrintf("%s\n", msg);
+		}
+	}
+	while (*pPlayback) {
+		pDev = *pPlayback++;
+		if (pDev->driver == DEV_DRIVER_PORTAUDIO) {
+			strMcpy(pDev->dev_errmsg, msg, QUISK_SC_SIZE);
+			if (quisk_sound_state.verbose_sound)
+				QuiskPrintf("%s\n", msg);
+		}
+	}
 }
 
-void quisk_close_sound_alsa(struct sound_dev ** pCapture, struct sound_dev ** pPlayback)
-{
-}
-
-int  quisk_read_alsa(struct sound_dev * dev, complex double * samp)
+int quisk_read_portaudio(struct sound_dev * dev, complex double * cSamples)
 {
 	return 0;
 }
 
-void quisk_mixer_set(char * card_name, int numid, PyObject * value, char * err_msg, int err_size)
-{
-    err_msg[0] = 0;
-}
+void quisk_play_portaudio(struct sound_dev * playdev, int nSamples, complex double * cSamples, int report_latency, double volume)
+{}
 
-#if !defined USE_MACPORTS
-int  quisk_read_pulseaudio(struct sound_dev * dev, complex double * samp)
-{
-	return 0;
-}
-
-void quisk_play_pulseaudio(struct sound_dev * dev, int j, complex double * samp, int i, double volume)
-{
-}
-
-void quisk_start_sound_pulseaudio(struct sound_dev ** pCapture, struct sound_dev ** pPlayback)
-{
-}
-
-void quisk_close_sound_pulseaudio()
-{
-}
-
-
-PyObject * quisk_pa_sound_devices(PyObject * self, PyObject * args)
-{	// Return a list of PulseAudio device names [pycapt, pyplay]
-	PyObject * pylist, * pycapt, * pyplay;
-
-	if (!PyArg_ParseTuple (args, ""))
-		return NULL;
-	pylist = PyList_New(0);		// list [pycapt, pyplay]
-	pycapt = PyList_New(0);		// list of capture devices
-	pyplay = PyList_New(0);		// list of play devices
-	PyList_Append(pylist, pycapt);
-	PyList_Append(pylist, pyplay);
-	return pylist;
-}
-#endif
+void quisk_close_sound_portaudio(void)
+{}
 #endif
