@@ -86,7 +86,7 @@ char * QuiskGetConfigString(const char * name, char * deflt)
 #else
     if (PyUnicode_Check(attr)) {
       PyObject * pystr = PyUnicode_AsUTF8String(attr);
-      strncpy(retbuf, PyString_AsString(pystr), QUISK_SC_SIZE);
+      strMcpy(retbuf, PyString_AsString(pystr), QUISK_SC_SIZE);
       retbuf[QUISK_SC_SIZE - 1] = 0;
       res = retbuf;
       Py_DECREF(pystr);
@@ -129,10 +129,22 @@ int QuiskDeltaMsec(int timer)
 {  // return the number of milliseconds since the last call for the timer.
    // There are two timers. The "timer" is either 0 or 1. Call first and throw away the result.
 	static long long time0[2] = {0, 0};
-	long long now;
+	long long now;  // in msec
 	int delta;
 #ifdef MS_WINDOWS
-	now = GetTickCount();	// This overflows after 49.7 days
+	// Code contributed by Ben Cahill
+	static long long timer_rate = 0;
+	LARGE_INTEGER L;
+	if ( ! timer_rate) {
+		if (QueryPerformanceFrequency(&L))
+			timer_rate = L.QuadPart;
+		else
+			timer_rate = 1;
+	}
+	if (QueryPerformanceCounter(&L))
+		now = (double)L.QuadPart * 1000 / timer_rate;
+	else
+		now = 0;
 #else
 	struct timespec ts;
 #ifdef CLOCK_MONOTONIC_RAW
@@ -191,18 +203,18 @@ void QuiskPrintTime(const char * str, int index)
 	// print the time since startup, and the time since the last call
 	if (index > 0) {
 		if (str[0])	// print message and a newline
-			printf ("%12.6lf  %9.3lf  %9.3lf  %s\n",
+			QuiskPrintf ("%12.6lf  %9.3lf  %9.3lf  %s\n",
 				tm - time0, (tm - start_time[0])*1e3, (tm - start_time[index])*1e3, str);
 		else		// no message; omit newline
-			printf ("%12.6lf  %9.3lf  %9.3lf  ",
+			QuiskPrintf ("%12.6lf  %9.3lf  %9.3lf  ",
 				tm - time0, (tm - start_time[0])*1e3, (tm - start_time[index])*1e3);
 	}
 	else {
 		if (str[0])	// print message and a newline
-			printf ("%12.6lf  %9.3lf  %s\n",
+			QuiskPrintf ("%12.6lf  %9.3lf  %s\n",
 				tm - time0, (tm - start_time[0])*1e3, str);
 		else		// no message; omit newline
-			printf ("%12.6lf  %9.3lf  ",
+			QuiskPrintf ("%12.6lf  %9.3lf  ",
 				tm - time0, (tm - start_time[0])*1e3);
 	}
 	start_time[0] = tm;
@@ -223,8 +235,8 @@ void QuiskSleepMicrosec(int usec)
 #endif
 }
 
-void QuiskMeasureRate(const char * msg, int count, int index)
-{  //measure the sample rate for index 0 to 9
+void QuiskMeasureRate(const char * msg, int count, int index, int reset)
+{  //measure the sample rate for index 0 to 9. If reset, reset the count and time at each print.
 	double tm;
 	static unsigned long total[10] = {0,0,0,0,0,0,0,0,0,0};
 	static double time0[10] = {0,0,0,0,0,0,0,0,0,0};
@@ -232,6 +244,7 @@ void QuiskMeasureRate(const char * msg, int count, int index)
 
 	if ( ! msg) {	// init
 		time0[index] = 0;
+                total[index] = 0;
 		return;
 	}
 	if (count && time0[index] == 0) {		// init
@@ -243,8 +256,67 @@ void QuiskMeasureRate(const char * msg, int count, int index)
 		return;
 	total[index] += count;
 	tm = QuiskTimeSec();
-	if (tm > time_pr[index] + 1.0) {	// time to print
+	if (tm > time_pr[index] + 10.0) {	// time to print
 		time_pr[index] = tm;
-		printf("%s count %ld, time %.3lf, rate %.3lf\n", msg, total[index], tm - time0[index], total[index] / (tm - time0[index]));
+		QuiskPrintf("%s count %ld, time %.3lf, rate %.3lf\n", msg, total[index], tm - time0[index], total[index] / (tm - time0[index]));
+                if (reset) {
+                        total[index] = 0;
+                        time0[index] = tm;
+                }
 	}
 }
+
+char * strMcpy(char * pDest, const char * pSrc, size_t sizeDest)
+{  // replacement for strncpy()
+	size_t sizeCopy;
+
+	sizeCopy = strnlen(pSrc, sizeDest - 1);
+	memcpy(pDest, pSrc, sizeCopy);
+	pDest[sizeCopy] = 0;
+	return pDest;
+}
+
+#ifdef MS_WINDOWS
+#define QP_BUF_DELTA   256
+PyObject * QuiskPrintf(char * format, ...)
+{
+        int length;
+        PyObject * py_string;
+        static int buf_size;            // size of buffer
+        static int buf_strlen;          // number of chars in buffer
+        static char * buffer = NULL;
+        va_list args;
+
+        EnterCriticalSection(&QuiskCriticalSection);
+        if (buffer == NULL) {   // initialize
+                buf_strlen = 0;
+                buf_size = QP_BUF_DELTA * 2;
+                buffer = malloc(buf_size);
+        }
+        if (format == NULL) {           // return the string
+                py_string = PyUnicode_DecodeUTF8(buffer, buf_strlen, "replace");
+                buf_strlen = 0;
+                LeaveCriticalSection(&QuiskCriticalSection);
+                return py_string;
+        }
+        if (buf_size - buf_strlen < QP_BUF_DELTA * 2) {     // max addition is QP_BUF_DELTA
+                buf_size += QP_BUF_DELTA * 2;
+                buffer = realloc(buffer, buf_size);
+        }
+        va_start(args, format);
+        length = vsnprintf(buffer + buf_strlen, QP_BUF_DELTA, format, args);
+        if (length < 0) {
+                strcpy(buffer + buf_strlen, "Encoding error\n");
+                buf_strlen = strlen(buffer);
+        }
+        else if (length > QP_BUF_DELTA - 1) {
+                buf_strlen = strlen(buffer);
+        }
+        else {
+                buf_strlen += length;
+        }
+        va_end(args);
+        LeaveCriticalSection(&QuiskCriticalSection);
+        return NULL;
+}
+#endif
